from belgie_alchemy.sso.adapter import SSOAdapter
from belgie_alchemy.sso.mixins import SSODomainMixin, SSOProviderMixin

__all__ = [
    "SSOAdapter",
    "SSODomainMixin",
    "SSOProviderMixin",
]



from __future__ import annotations

from datetime import UTC, datetime
from typing import TYPE_CHECKING

from belgie_proto.organization.invitation import InvitationProtocol
from belgie_proto.organization.member import MemberProtocol
from belgie_proto.organization.organization import OrganizationProtocol
from belgie_proto.team import TeamAdapterProtocol
from belgie_proto.team.member import TeamMemberProtocol
from belgie_proto.team.team import TeamProtocol
from sqlalchemy import delete, select, update

from belgie_alchemy.organization.adapter import OrganizationAdapter

if TYPE_CHECKING:
    from uuid import UUID

    from belgie_proto.core.connection import DBConnection


class TeamAdapter[
    OrganizationT: OrganizationProtocol,
    MemberT: MemberProtocol,
    InvitationT: InvitationProtocol,
    TeamT: TeamProtocol,
    TeamMemberT: TeamMemberProtocol,
](
    OrganizationAdapter[OrganizationT, MemberT, InvitationT],
    TeamAdapterProtocol[OrganizationT, MemberT, InvitationT, TeamT, TeamMemberT],
):
    def __init__(
        self,
        *,
        organization: type[OrganizationT],
        member: type[MemberT],
        invitation: type[InvitationT],
        team: type[TeamT],
        team_member: type[TeamMemberT],
    ) -> None:
        super().__init__(organization=organization, member=member, invitation=invitation)
        self.team_model = team
        self.team_member_model = team_member

    async def remove_member(
        self,
        session: DBConnection,
        *,
        organization_id: UUID,
        user_id: UUID,
    ) -> bool:
        team_ids_stmt = select(self.team_model.id).where(self.team_model.organization_id == organization_id)
        await session.execute(
            delete(self.team_member_model).where(
                self.team_member_model.user_id == user_id,
                self.team_member_model.team_id.in_(team_ids_stmt),
            ),
        )
        member_delete_result = await session.execute(
            delete(self.member_model).where(
                self.member_model.organization_id == organization_id,
                self.member_model.user_id == user_id,
            ),
        )
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return member_delete_result.rowcount > 0  # type: ignore[attr-defined]

    async def create_team(
        self,
        session: DBConnection,
        *,
        organization_id: UUID,
        name: str,
    ) -> TeamT:
        team = self.team_model(
            organization_id=organization_id,
            name=name,
        )
        session.add(team)
        try:
            await session.commit()
            await session.refresh(team)
        except Exception:
            await session.rollback()
            raise
        return team

    async def get_team_by_id(
        self,
        session: DBConnection,
        team_id: UUID,
    ) -> TeamT | None:
        stmt = select(self.team_model).where(self.team_model.id == team_id)
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def list_teams(
        self,
        session: DBConnection,
        *,
        organization_id: UUID,
    ) -> list[TeamT]:
        stmt = select(self.team_model).where(self.team_model.organization_id == organization_id)
        result = await session.execute(stmt)
        return list(result.scalars().all())

    async def update_team(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
        name: str,
    ) -> TeamT | None:
        stmt = (
            update(self.team_model)
            .where(self.team_model.id == team_id)
            .values(name=name, updated_at=datetime.now(UTC))
            .returning(self.team_model)
        )
        try:
            result = await session.execute(stmt)
            row = result.scalar_one_or_none()
            # If RETURNING is empty, still commit: rollback would drop unrelated pending ORM
            # work that may exist on the same session alongside this call.
            await session.commit()
            if row is None:
                return None
            await session.refresh(row)
        except Exception:
            await session.rollback()
            raise
        return row

    async def remove_team(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
    ) -> bool:
        stmt = delete(self.team_model).where(self.team_model.id == team_id)
        result = await session.execute(stmt)
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return result.rowcount > 0  # type: ignore[attr-defined]

    async def add_team_member(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
        user_id: UUID,
    ) -> TeamMemberT:
        team_member = self.team_member_model(
            team_id=team_id,
            user_id=user_id,
        )
        session.add(team_member)
        try:
            await session.commit()
            await session.refresh(team_member)
        except Exception:
            await session.rollback()
            raise
        return team_member

    async def remove_team_member(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
        user_id: UUID,
    ) -> bool:
        stmt = delete(self.team_member_model).where(
            self.team_member_model.team_id == team_id,
            self.team_member_model.user_id == user_id,
        )
        result = await session.execute(stmt)
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return result.rowcount > 0  # type: ignore[attr-defined]

    async def get_team_member(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
        user_id: UUID,
    ) -> TeamMemberT | None:
        stmt = select(self.team_member_model).where(
            self.team_member_model.team_id == team_id,
            self.team_member_model.user_id == user_id,
        )
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def list_team_members(
        self,
        session: DBConnection,
        *,
        team_id: UUID,
    ) -> list[TeamMemberT]:
        stmt = select(self.team_member_model).where(self.team_member_model.team_id == team_id)
        result = await session.execute(stmt)
        return list(result.scalars().all())

    async def list_teams_for_user(
        self,
        session: DBConnection,
        *,
        user_id: UUID,
    ) -> list[TeamT]:
        stmt = (
            select(self.team_model)
            .join(
                self.team_member_model,
                self.team_member_model.team_id == self.team_model.id,
            )
            .where(self.team_member_model.user_id == user_id)
        )
        result = await session.execute(stmt)
        return list(result.scalars().all())



from belgie_alchemy.core.adapter import BelgieAdapter
from belgie_alchemy.core.mixins import AccountMixin, OAuthStateMixin, SessionMixin, UserMixin

__all__ = [
    "AccountMixin",
    "BelgieAdapter",
    "OAuthStateMixin",
    "SessionMixin",
    "UserMixin",
]



from __future__ import annotations

from datetime import UTC, datetime
from typing import TYPE_CHECKING

from belgie_proto.sso import SSOAdapterProtocol
from belgie_proto.sso.provider import OIDCConfigValue, SSOProviderProtocol
from sqlalchemy import delete, select

if TYPE_CHECKING:
    from uuid import UUID

    from belgie_proto.core.connection import DBConnection
    from belgie_proto.sso.domain import SSODomainProtocol


class SSOAdapter[
    ProviderT: SSOProviderProtocol,
    DomainT: SSODomainProtocol,
](SSOAdapterProtocol[ProviderT, DomainT]):
    def __init__(
        self,
        *,
        sso_provider: type[ProviderT],
        sso_domain: type[DomainT],
    ) -> None:
        self.sso_provider_model = sso_provider
        self.sso_domain_model = sso_domain

    async def create_provider(
        self,
        session: DBConnection,
        *,
        organization_id: UUID,
        provider_id: str,
        issuer: str,
        oidc_config: dict[str, OIDCConfigValue],
    ) -> ProviderT:
        provider = self.sso_provider_model(
            organization_id=organization_id,
            provider_id=provider_id,
            issuer=issuer,
            oidc_config=oidc_config,
        )
        session.add(provider)
        try:
            await session.commit()
            await session.refresh(provider)
        except Exception:
            await session.rollback()
            raise
        return provider

    async def get_provider_by_id(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
    ) -> ProviderT | None:
        stmt = select(self.sso_provider_model).where(self.sso_provider_model.id == sso_provider_id)
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def get_provider_by_provider_id(
        self,
        session: DBConnection,
        *,
        provider_id: str,
    ) -> ProviderT | None:
        stmt = select(self.sso_provider_model).where(self.sso_provider_model.provider_id == provider_id)
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def list_providers_for_organization(
        self,
        session: DBConnection,
        *,
        organization_id: UUID,
    ) -> list[ProviderT]:
        stmt = select(self.sso_provider_model).where(self.sso_provider_model.organization_id == organization_id)
        result = await session.execute(stmt)
        return list(result.scalars().all())

    async def update_provider(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
        issuer: str | None = None,
        oidc_config: dict[str, OIDCConfigValue] | None = None,
    ) -> ProviderT | None:
        provider = await self.get_provider_by_id(session, sso_provider_id=sso_provider_id)
        if provider is None:
            return None

        if issuer is not None:
            provider.issuer = issuer
        if oidc_config is not None:
            provider.oidc_config = oidc_config
        provider.updated_at = datetime.now(UTC)

        try:
            await session.commit()
            await session.refresh(provider)
        except Exception:
            await session.rollback()
            raise
        return provider

    async def delete_provider(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
    ) -> bool:
        stmt = delete(self.sso_provider_model).where(self.sso_provider_model.id == sso_provider_id)
        result = await session.execute(stmt)
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return result.rowcount > 0  # type: ignore[attr-defined]

    async def create_domain(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
        domain: str,
        verification_token: str,
    ) -> DomainT:
        sso_domain = self.sso_domain_model(
            sso_provider_id=sso_provider_id,
            domain=domain,
            verification_token=verification_token,
        )
        session.add(sso_domain)
        try:
            await session.commit()
            await session.refresh(sso_domain)
        except Exception:
            await session.rollback()
            raise
        return sso_domain

    async def get_domain(
        self,
        session: DBConnection,
        *,
        domain_id: UUID,
    ) -> DomainT | None:
        stmt = select(self.sso_domain_model).where(self.sso_domain_model.id == domain_id)
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def get_domain_by_name(
        self,
        session: DBConnection,
        *,
        domain: str,
    ) -> DomainT | None:
        stmt = select(self.sso_domain_model).where(self.sso_domain_model.domain == domain)
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def get_verified_domain(
        self,
        session: DBConnection,
        *,
        domain: str,
    ) -> DomainT | None:
        stmt = select(self.sso_domain_model).where(
            self.sso_domain_model.domain == domain,
            self.sso_domain_model.verified_at.is_not(None),
        )
        result = await session.execute(stmt)
        return result.scalar_one_or_none()

    async def list_domains_for_provider(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
    ) -> list[DomainT]:
        stmt = select(self.sso_domain_model).where(self.sso_domain_model.sso_provider_id == sso_provider_id)
        result = await session.execute(stmt)
        return list(result.scalars().all())

    async def update_domain(
        self,
        session: DBConnection,
        *,
        domain_id: UUID,
        verification_token: str | None = None,
        verified_at: datetime | None = None,
    ) -> DomainT | None:
        sso_domain = await self.get_domain(session, domain_id=domain_id)
        if sso_domain is None:
            return None

        if verification_token is not None:
            sso_domain.verification_token = verification_token
        sso_domain.verified_at = verified_at
        sso_domain.updated_at = datetime.now(UTC)

        try:
            await session.commit()
            await session.refresh(sso_domain)
        except Exception:
            await session.rollback()
            raise
        return sso_domain

    async def delete_domain(
        self,
        session: DBConnection,
        *,
        domain_id: UUID,
    ) -> bool:
        stmt = delete(self.sso_domain_model).where(self.sso_domain_model.id == domain_id)
        result = await session.execute(stmt)
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return result.rowcount > 0  # type: ignore[attr-defined]

    async def delete_domains_for_provider(
        self,
        session: DBConnection,
        *,
        sso_provider_id: UUID,
    ) -> int:
        stmt = delete(self.sso_domain_model).where(self.sso_domain_model.sso_provider_id == sso_provider_id)
        result = await session.execute(stmt)
        try:
            await session.commit()
        except Exception:
            await session.rollback()
            raise
        return result.rowcount  # type: ignore[attr-defined]



from datetime import datetime
from uuid import UUID

from brussels.types import DateTimeUTC, Json
from sqlalchemy import ForeignKey, Index, Text, UniqueConstraint
from sqlalchemy.dialects.postgresql import CITEXT
from sqlalchemy.orm import Mapped, MappedAsDataclass, declarative_mixin, declared_attr, mapped_column, relationship


@declarative_mixin
class SSOProviderMixin(MappedAsDataclass):
    __tablename__ = "sso_provider"

    @declared_attr
    def organization_id(self) -> Mapped[UUID]:
        return mapped_column(
            ForeignKey("organization.id", ondelete="cascade", onupdate="cascade"),
            nullable=False,
            kw_only=True,
        )

    @declared_attr
    def provider_id(self) -> Mapped[str]:
        provider_type = Text().with_variant(CITEXT(), "postgresql")
        return mapped_column(provider_type, unique=True, index=True, kw_only=True)

    @declared_attr
    def issuer(self) -> Mapped[str]:
        issuer_type = Text().with_variant(CITEXT(), "postgresql")
        return mapped_column(issuer_type, index=True, kw_only=True)

    @declared_attr
    def oidc_config(self) -> Mapped[dict[str, str | list[str] | dict[str, str]]]:
        return mapped_column(Json, kw_only=True)

    @declared_attr
    def domains(self) -> Mapped[list[object]]:
        return relationship(
            "SSODomain",
            back_populates="provider",
            cascade="all, delete-orphan",
            init=False,
        )

    @declared_attr.directive
    def __table_args__(self) -> tuple[Index]:
        return (
            Index(
                "ix_sso_provider_organization_id",
                self.organization_id,
            ),
        )


@declarative_mixin
class SSODomainMixin(MappedAsDataclass):
    __tablename__ = "sso_domain"

    @declared_attr
    def sso_provider_id(self) -> Mapped[UUID]:
        return mapped_column(
            ForeignKey("sso_provider.id", ondelete="cascade", onupdate="cascade"),
            nullable=False,
            kw_only=True,
        )

    @declared_attr
    def domain(self) -> Mapped[str]:
        domain_type = Text().with_variant(CITEXT(), "postgresql")
        return mapped_column(domain_type, unique=True, index=True, kw_only=True)

    @declared_attr
    def verification_token(self) -> Mapped[str]:
        return mapped_column(Text, kw_only=True)

    @declared_attr
    def verified_at(self) -> Mapped[datetime | None]:
        return mapped_column(DateTimeUTC, default=None, kw_only=True)

    @declared_attr
    def provider(self) -> Mapped[object]:
        return relationship(
            "SSOProvider",
            back_populates="domains",
            lazy="selectin",
            init=False,
        )

    @declared_attr.directive
    def __table_args__(self) -> tuple[UniqueConstraint, Index]:
        return (
            UniqueConstraint("domain", name="uq_sso_domain_domain"),
            Index("ix_sso_domain_sso_provider_id", self.sso_provider_id),
        )


__all__ = [
    "SSODomainMixin",
    "SSOProviderMixin",
]



from __future__ import annotations

from datetime import datetime  # noqa: TC003
from uuid import UUID  # noqa: TC003

from brussels.types import DateTimeUTC
from sqlalchemy import JSON, ForeignKey, Index, Text, UniqueConstraint
from sqlalchemy.dialects.postgresql import ARRAY, CITEXT
from sqlalchemy.orm import Mapped, MappedAsDataclass, declarative_mixin, declared_attr, mapped_column, relationship


@declarative_mixin
class UserMixin(MappedAsDataclass):
    __tablename__ = "user"

    @declared_attr
    def email(self) -> Mapped[str]:
        email_type = Text().with_variant(CITEXT(), "postgresql")
        return mapped_column(email_type, unique=True, index=True, kw_only=True)

    @declared_attr
    def email_verified_at(self) -> Mapped[datetime | None]:
        return mapped_column(DateTimeUTC, default=None, kw_only=True)

    @declared_attr
    def name(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def image(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def scopes(self) -> Mapped[list[str]]:
        scopes_type = JSON().with_variant(ARRAY(Text()), "postgresql")
        return mapped_column(scopes_type, default_factory=list, nullable=False, kw_only=True)

    @declared_attr
    def accounts(self) -> Mapped[list[object]]:
        return relationship(
            "Account",
            back_populates="user",
            cascade="all, delete-orphan",
            init=False,
        )

    @declared_attr
    def sessions(self) -> Mapped[list[object]]:
        return relationship(
            "Session",
            back_populates="user",
            cascade="all, delete-orphan",
            init=False,
        )

    @declared_attr
    def oauth_states(self) -> Mapped[list[object]]:
        return relationship(
            "OAuthState",
            back_populates="user",
            cascade="all, delete-orphan",
            init=False,
        )


@declarative_mixin
class AccountMixin(MappedAsDataclass):
    __tablename__ = "account"

    @declared_attr
    def user_id(self) -> Mapped[UUID]:
        return mapped_column(
            ForeignKey("user.id", ondelete="cascade", onupdate="cascade"),
            nullable=False,
            kw_only=True,
        )

    @declared_attr
    def provider(self) -> Mapped[str]:
        provider_type = Text().with_variant(CITEXT(), "postgresql")
        return mapped_column(provider_type, kw_only=True)

    @declared_attr
    def provider_account_id(self) -> Mapped[str]:
        return mapped_column(Text(), kw_only=True)

    @declared_attr
    def access_token(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def refresh_token(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def expires_at(self) -> Mapped[datetime | None]:
        return mapped_column(DateTimeUTC, default=None, kw_only=True)

    @declared_attr
    def token_type(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def scope(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def id_token(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def user(self) -> Mapped[object]:
        return relationship(
            "User",
            back_populates="accounts",
            lazy="selectin",
            init=False,
        )

    @declared_attr.directive
    def __table_args__(self) -> tuple[UniqueConstraint, Index]:
        return (
            UniqueConstraint(
                self.provider,
                self.provider_account_id,
                name="uq_accounts_provider_provider_account_id",
            ),
            Index(
                f"ix_{self.__tablename__}_user_id_provider",
                self.user_id,
                self.provider,
            ),
        )


@declarative_mixin
class SessionMixin(MappedAsDataclass):
    __tablename__ = "session"

    @declared_attr
    def user_id(self) -> Mapped[UUID]:
        return mapped_column(
            ForeignKey("user.id", ondelete="cascade", onupdate="cascade"),
            index=True,
            nullable=False,
            kw_only=True,
        )

    @declared_attr
    def expires_at(self) -> Mapped[datetime]:
        return mapped_column(DateTimeUTC, index=True, kw_only=True)

    @declared_attr
    def ip_address(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def user_agent(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def user(self) -> Mapped[object]:
        return relationship(
            "User",
            back_populates="sessions",
            lazy="selectin",
            init=False,
        )


@declarative_mixin
class OAuthStateMixin(MappedAsDataclass):
    __tablename__ = "oauth_state"

    @declared_attr
    def state(self) -> Mapped[str]:
        return mapped_column(Text, unique=True, index=True, kw_only=True)

    @declared_attr
    def user_id(self) -> Mapped[UUID | None]:
        return mapped_column(
            ForeignKey("user.id", ondelete="set null", onupdate="cascade"),
            nullable=True,
            kw_only=True,
        )

    @declared_attr
    def expires_at(self) -> Mapped[datetime]:
        return mapped_column(DateTimeUTC, kw_only=True)

    @declared_attr
    def code_verifier(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def redirect_url(self) -> Mapped[str | None]:
        return mapped_column(Text, default=None, kw_only=True)

    @declared_attr
    def user(self) -> Mapped[object | None]:
        return relationship(
            "User",
            back_populates="oauth_states",
            lazy="selectin",
            init=False,
        )


__all__ = [
    "AccountMixin",
    "OAuthStateMixin",
    "SessionMixin",
    "UserMixin",
]
